import re
from dataclasses import dataclass
from typing import Callable


@dataclass
class RegexSpmMatch:
  string: str
  _match_func: Callable[[re.Pattern, str], re.Match]
  match: re.Match | None = None

  def __eq__(self, pattern: str | re.Pattern | tuple[str, int | re.RegexFlag]):
    if isinstance(pattern, str):
      pattern = re.compile(pattern)
    elif isinstance(pattern, tuple):
      pattern = re.compile(*pattern)
    self.match = self._match_func(pattern, self.string)
    return self.match is not None

  def __getitem__(
      self,
      group: int | str | tuple[int, ...] | tuple[str, ...]
  ) -> str | tuple[str, ...] | None:
    return self.match[group]



import re

from .regex_spm_match import RegexSpmMatch


def search_in(string: str) -> RegexSpmMatch:
  return RegexSpmMatch(string, _match_func=re.search)


def match_in(string: str) -> RegexSpmMatch:
  return RegexSpmMatch(string, _match_func=re.match)


def fullmatch_in(string: str) -> RegexSpmMatch:
  return RegexSpmMatch(string, _match_func=re.fullmatch)


__all__ = ['RegexSpmMatch', 'search_in', 'match_in', 'fullmatch_in']



from setuptools import setup

with open('README.md', 'r') as f:
  long_description = f.read()

setup(
  name='regex_spm',
  version='1.0.0',
  packages=['regex_spm'],
  url='https://github.com/aronhoff/regex_spm',
  license='MIT',
  author='aronhoffmann',
  author_email='aron.m.hoff+pypy@gmail.com',
  description='Enable Structural Pattern Matching for Python regular expressions',
  long_description=long_description,
  long_description_content_type='text/markdown',
  classifiers=[
    'Programming Language :: Python :: 3',
    'License :: OSI Approved :: MIT License',
    'Operating System :: OS Independent',
    'Topic :: Software Development :: Libraries :: Python Modules',
  ],
  python_requires='>=3.10',
)
